package Project::Euler;

use warnings;
use strict;

=head1 NAME

Project::Euler - Solutions for projecteuler.net

=head1 VERSION

Version 0.1.3

=cut

use version 0.77; our $VERSION = qv("v0.1.3");


=head1 SYNOPSIS

This is the base class which will eventually be responsible for displaying the
interface to interact with the solutions implemented so far

    use Project::Euler;

=head1 EXPORT

#  Todo:  implement interface functions

=head1 FUNCTIONS

=head2 show_solutions

=cut

sub show_solutions {
    print "There aren't any yet!\n";
}


=head1 AUTHOR

Adam Lesperance, C<< <lespea at cpan.org> >>

=head1 BUGS

Please report any bugs or feature requests to C<bug-project-euler at rt.cpan.org>, or through
the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Project-Euler>.  I will be notified, and then you'll
automatically be notified of progress on your bug as I make changes.




=head1 SUPPORT

You can find documentation for this module with the perldoc command.

    perldoc Project::Euler


You can also look for information at:

=over 4

=item * RT: CPAN's request tracker

L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Project-Euler>

=item * AnnoCPAN: Annotated CPAN documentation

L<http://annocpan.org/dist/Project-Euler>

=item * CPAN Ratings

L<http://cpanratings.perl.org/d/Project-Euler>

=item * Search CPAN

L<http://search.cpan.org/dist/Project-Euler/>

=back


=head1 ACKNOWLEDGEMENTS


=head1 COPYRIGHT & LICENSE

Copyright 2009 Adam Lesperance.

This program is free software; you can redistribute it and/or modify it
under the terms of either: the GNU General Public License as published
by the Free Software Foundation; or the Artistic License.

See http://dev.perl.org/licenses/ for more information.


=cut

1; # End of Project::Euler
